﻿using System;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace CSHowDoISamples
{
    public class MyTextStyle : TextStyle
    {
        protected override void DrawCore(System.Collections.Generic.IEnumerable<Feature> features, GeoCanvas canvas, Collection<SimpleCandidate> labelsInThisLayer, Collection<SimpleCandidate> labelsInAllLayers)
        {
            // This only works for Convex polygon and the vertex order for line is anticlockwise
            foreach (Feature f in features)
            {
                LineShape line = f.GetShape() as LineShape;

                ScreenPointF spf = ExtentHelper.ToScreenCoordinate(canvas.CurrentWorldExtent, line.GetCenterPoint(), canvas.Width, canvas.Height); // Center point of current line                

                float rotateAngle = CalculateRotateAngleByLine(line);

                float yoffset = 50;

                if (line.Vertices[1].X < line.Vertices[0].X)
                {
                    yoffset = -50;
                }

                canvas.DrawText(f.ColumnValues["label"], base.Font, base.TextSolidBrush, base.HaloPen, new ScreenPointF[] { spf }, ThinkGeo.MapSuite.Core.DrawingLevel.LabelLevel, 0, yoffset, rotateAngle);
            }
        }

        private float CalculateRotateAngleByLine(LineShape line)
        {
            float rotateAngle = 0;

            // Please implement your algorithm here

            return rotateAngle;
        }
    }

    public partial class DisplayASimpleMap : UserControl
    {
        public DisplayASimpleMap()
        {
            InitializeComponent();
        }

        private void WpfMap_Loaded(object sender, RoutedEventArgs e)
        {
            Map1.MapUnit = GeographyUnit.DecimalDegree;

            GeoFont font = new GeoFont("Arial", 20, DrawingFontStyles.Regular);
            GeoSolidBrush txtBrush = new GeoSolidBrush(GeoColor.StandardColors.Black);

            MyTextStyle textStyle = new MyTextStyle();
            textStyle.TextColumnName = "label";
            textStyle.Font = font;
            textStyle.TextSolidBrush = txtBrush;
            
            InMemoryFeatureLayer layer = new InMemoryFeatureLayer();
            layer.Open();
            layer.Columns.Add(new FeatureSourceColumn("label"));
            layer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = textStyle;
            layer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.YOffsetInPixel = 10;
            layer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.Black, 1, true);
            layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LineShape line1 = new LineShape();
            line1.Vertices.Add(new Vertex(0, 0));
            line1.Vertices.Add(new Vertex(3, -2));
            Feature f1 = new Feature(line1);
            f1.ColumnValues.Add("label", "173");

            LineShape line2 = new LineShape();
            line2.Vertices.Add(new Vertex(3, -2));
            line2.Vertices.Add(new Vertex(6, 5));
            Feature f2 = new Feature(line2);
            f2.ColumnValues.Add("label", "192");


            LineShape line3 = new LineShape();
            line3.Vertices.Add(new Vertex(6, 5));
            line3.Vertices.Add(new Vertex(3, 9));
            Feature f3 = new Feature(line3);
            f3.ColumnValues.Add("label", "194");


            LineShape line4 = new LineShape();
            line4.Vertices.Add(new Vertex(3, 9));
            line4.Vertices.Add(new Vertex(-2, 7));
            Feature f4 = new Feature(line4);
            f4.ColumnValues.Add("label", "196");


            LineShape line5 = new LineShape();
            line5.Vertices.Add(new Vertex(-2, 7));
            line5.Vertices.Add(new Vertex(-4, -2));
            Feature f5 = new Feature(line5);
            f5.ColumnValues.Add("label", "201");


            LineShape line6 = new LineShape();
            line6.Vertices.Add(new Vertex(-4, -2));
            line6.Vertices.Add(new Vertex(0, 0));
            Feature f6 = new Feature(line6);
            f6.ColumnValues.Add("label", "172");


            layer.InternalFeatures.Add(f1);
            layer.InternalFeatures.Add(f2);
            layer.InternalFeatures.Add(f3);
            layer.InternalFeatures.Add(f4);
            layer.InternalFeatures.Add(f5);
            layer.InternalFeatures.Add(f6);

            layer.Open();
            Map1.CurrentExtent = layer.GetBoundingBox();

            LayerOverlay overlay = new LayerOverlay();
            overlay.Layers.Add(layer);

            Map1.Overlays.Add(overlay);

            Map1.Refresh();
        }

        private void btnTest_Click(object sender, RoutedEventArgs e)
        {
            ShapeFileFeatureLayer layer = (Map1.Overlays[0] as LayerOverlay).Layers[0] as ShapeFileFeatureLayer;

            if (!layer.IsOpen)
            {
                layer.Open();
            }

            Collection<Feature> features = layer.QueryTools.GetFeaturesNearestTo(new PointShape(20, -40), GeographyUnit.DecimalDegree, 5, ReturningColumnsType.AllColumns);
            resultLabel.Text = features.Count.ToString();

            Map1.CenterAt(new PointShape(20, -40));
            Map1.Refresh();
        }
    }
}